<?php
/**
 * @file
 * Builds the page for the path node/%/robotagger.
 */

/**
 * Returns the output to confirm/decline annotation suggestion and
 * send manually send a request to the webservice.
 *
 * @param stdClass $node
 *   The node object.
 *
 * @return string $output
 *   The rendered page.
 */
function robotagger_node_page($node) {
  $output = '';
  $suggestions = _robotagger_get_suggestions($node);
  $declined = _robotagger_get_declined_suggestions($node);
  if (!empty($suggestions) || !empty($declined)) {
    $output .= drupal_get_form('robotagger_node_page_form', $node, $suggestions, $declined);
  }
  $settings = _robotagger_get_settings($node->type);
  if ($settings->sendrequest == 'manual') {
    $output .= drupal_get_form('robotagger_sendrequest_form', $node);
  }
  return $output;
}

/**
 * Builds the form where the user can confirm/decline annotation suggestions.
 *
 * @param array $form_state
 * @param array $node
 *   The node.
 * @param array $suggestions
 *   An array of suggestions.
 * @param array $declined
 *   An array of declined suggestions.
 *
 * @see _robotagger_get_suggestions().
 * @see _robotagger_get_declined_suggestions().
 *
 */
function robotagger_node_page_form($form_state, $node, $suggestions = array(), $declined = array()) {
  $form = array('#tree' => TRUE);
  $form['#node'] = $node;
  if (!empty($suggestions)) {
    $form['vids']['caption'] = array(
      '#value' => '<h2>' . t('Suggestions') . '</h2>',
    );
    foreach ($suggestions as $vid => $suggestion) {
      $voc = taxonomy_vocabulary_load($vid);
      $options[$vid] = $suggestion;
      $form['vids'][$vid] = array(
        '#type' => 'checkboxes',
        '#options' => $options[$vid],
        '#title' => $voc->name,
      );
    }
  }
  if (!empty($declined)) {
    $form['declined']['caption'] = array(
      '#value' => '<h2>' . t('Declined suggestions') . '</h2>',
    );
    $options = array();
    foreach ($declined as $vid => $suggestions) {
      $voc = taxonomy_vocabulary_load($vid);
      $options[$vid] = $suggestions;
      $form['declined'][$vid] = array(
        '#type' => 'checkboxes',
        '#options' => $options[$vid],
        '#title' => $voc->name,
      );
    }
  }
  $form['action'] = array(
    '#type' => 'select',
    '#options' => array('confirm' => t('Confirm'), 'decline' => t('Decline')),
  );
  $form['button'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );
  return $form;
}

/**
 * Submit handler for our confirmation form.
 *
 * @param array $form
 * @param array $form_state
 *
 * @see robotagger_node_page_form().
 */
function robotagger_node_page_form_submit($form, &$form_state) {
  include_once dirname(__FILE__).'/robotagger.process.inc';
  $node = $form['#node'];
  $node->robotagger_process_all = FALSE;
  $node->robotagger_suggestions = array();
  $newsuggestions = array();
  if (empty($form_state['values']['vids'])) {
    $form_state['values']['vids'] = array();
  }
  if (empty($form_state['values']['declined'])) {
    $form_state['values']['declined'] = array();
  }
  $submitted = $form_state['values']['vids'] + $form_state['values']['declined'];
  if (!empty($submitted)) {
    foreach ($submitted as $vid => $annotypes) {
      $annotypes = array_filter($annotypes);
      if (!empty($annotypes)) {
        $newsuggestions[$vid] = $annotypes;
      }
    }
    $node->robotagger_suggestions = $newsuggestions;
  }
  if (!empty($node->robotagger_suggestions)) {
    $form_state['redirect'] = 'node/' . $node->nid;
    if ($form_state['values']['action'] == 'confirm') {
      node_save($node);
      drupal_set_message(t('The annotation suggestion have been confirmed.'));
      return;
    }
    elseif ($form_state['values']['action'] == 'decline') {
      drupal_set_message(t('The annotation suggestion have been declined.'));
      _robotagger_suggestions_decline($node);
      return;
    }
  }
}

/**
 * Builds the form with a button to requests the web service manually.
 */
function robotagger_sendrequest_form($form_state, $node) {
  $form['#node'] = $node;
  $form['desc'] = array(
    '#type' => 'item',
    '#value' => t('Click the button to send your current content to the RoboTagger web service.'),
  );
  $form['button'] = array(
    '#type' => 'submit',
    '#value' => t('Send Request'),
  );
  return $form;
}

/**
 * Submit handler for robotagger_sendrequest_form().
 */
function robotagger_sendrequest_form_submit($form, $form_state) {
  $node = (object) $form['#node'];
  $node->rtcheck_sendrequest = FALSE;
  node_save($node);
}
